////////////////////////////////////////////////////////////////////////////////
//      _    _  ____            ____    // Acquire.pas: unit Acquire          //
//     / \  | |/ ___| ___ _ __ |___ \   //------------------------------------//
//    / _ \ | | |  _ / _ \ '_ \  __) |  // Prozedur AcquireImage und dazu-    //
//   / ___ \| | |_| |  __/ | | |/ __/   // gehrige Toolroutinen zum Einlesen //
//  /_/   \_\_|\____|\___|_| |_|_____|  // eines Bildes in die Bilderliste    //
//                                      //                                    //
////////////////////////////////////////////////////////////////////////////////
unit Acquire;
interface
uses Datacore;

// Return-Flag: gibt an, ob die letzte AcquireImage-Operation gescheitert ist,
// weil die Dateiendung nicht erkannt wurde
var AI_Unknown_File_Type:boolean;

function AcquireImage(const _PathName:string):boolean;
                                                    // Bild zur Liste hinzufgen

implementation
uses SysUtils, DateUtils, Classes, Graphics, JPEG, EXIF, Main;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////// Tool-Routinen
////////////////////////////////////////////////////////////////////////////////

///// CreateThumbnail: Erzeugt ein Bitmap der Gre Datacore.ThumbnailSize[XY]
///// und pat dort skaliert das Source-Bitmap ein
function CreateThumbnail(const Source:TBitmap):TBitmap;
var OldX,OldY,NewX,NewY,RefX,RefY:integer;
begin
  // Bitmap erzeugen
  Result:=TBitmap.Create;
  with Result do begin
    PixelFormat:=pfDevice;
    Width:=ThumbnailWidth;
    Height:=ThumbnailHeight;
  end;
  // Zielgre berechnen
  OldX:=Source.Width;
  OldY:=Source.Height;
  if OldX>ThumbnailWidth then begin
    NewX:=ThumbnailWidth;
    NewY:=NewX*OldY DIV OldX;
  end else begin
    NewX:=OldX;
    NewY:=OldY;
  end;
  if NewY>ThumbnailHeight then begin
    NewY:=ThumbnailHeight;
    NewX:=NewY*OldX DIV OldY;
  end;
  // Linken oberen Punkt berechnen und Bitmap auf Thumbnail zeichnen
  RefX:=(ThumbnailWidth-NewX) DIV 2;
  RefY:=(ThumbnailHeight-NewY) DIV 2;
  Result.Canvas.StretchDraw(Rect(RefX,RefY,RefX+NewX,RefY+NewY),Source);
end;


///// ParseEXIFDate: nimmt einen EXIF-Datums/Zeitstring auseinander und
///// konstruiert ein Delphi-TDateTime daraus
function ParseEXIFDate(const DateStr:string; const Default:TDateTime):TDateTime;
begin
  try
    Result:=EncodeDateTime(
              StrToInt(copy(DateStr, 1,4)),
              StrToInt(copy(DateStr, 6,2)),
              StrToInt(copy(DateStr, 9,2)),
              StrToInt(copy(DateStr,12,2)),
              StrToInt(copy(DateStr,15,2)),
              StrToInt(copy(DateStr,18,2)),
            0);
  except
    Result:=Default;
  end;
end;


////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////// AcquireImage
////////////////////////////////////////////////////////////////////////////////

function AcquireImage(const _PathName:string):boolean;
var Ext:string;
    IsJPEG:boolean;
    JPEG:TJPEGImage;
    FullImage:TBitmap;
    Format,EXIFDateString,Camera:string;
begin
  Result:=false;  // gesunder Pessimismus schadet nie
  // Dateityp ermitteln
  Ext:=LowerCase(ExtractFileExt(_PathName));
  IsJPEG:=(Ext='.jpg') OR (Ext='.jpeg');
  AI_Unknown_File_Type:=not(IsJPEG) AND (Ext<>'.bmp');
  if AI_Unknown_File_Type then exit;
  // Standardwerte vorgeben
  FullImage:=TBitmap.Create;
  EXIFDateString:=''; Camera:='';
  if IsJPEG then begin
    ///// JPEG laden
    Format:='JPEG ';
    JPEG:=nil;
    try
      JPEG:=TJPEGImage.Create;
      JPEG.Performance:=jpBestSpeed;
      JPEG.LoadFromFile(_PathName);
      FullImage.Assign(JPEG);
      JPEG.Free;
    except
      if Assigned(JPEG) then JPEG.Free;
      FullImage.Free;
      exit;
    end;
    try
      // wichtige EXIF-Infos auslesen
      EXIFEssentials(_PathName,EXIFDateString,Camera);
    except
    end;
  end else begin
    ///// BMP laden
    Format:='BMP ';
    try
      FullImage.LoadFromFile(_PathName);
    except
      FullImage.Free;
      exit;
    end;
  end;
  // Bilderlisteneintrag hinzufgen
  with Data.AddImage()^ do begin
    Thumbnail:=CreateThumbnail(FullImage);
    FileName:=ExtractFileName(_PathName);
    PathName:=_PathName;
    ImageInfo:=Format+IntToStr(FullImage.Width)+'x'+IntToStr(FullImage.Height);
    if Camera<>'' then ImageInfo:=ImageInfo+' ('+Camera+')';
    Active:=true; Selected:=true;
    FileDate:=FileDateToDateTime(FileAge(_PathName));
    EXIFDate:=ParseEXIFDate(EXIFDateString,FileDate);
    DisplayDate:=FormatDateTime('dd.mm.yyyy hh:nn',EXIFDate);
    if Data.SelectedCategory>0
      then Category:=Data.SelectedCategory
      else Category:=0;
  end;
  FullImage.Free;
  Result:=true;
end;

end.
