////////////////////////////////////////////////////////////////////////////////
//      _    _  ____            ____    // Datacore.pas: unit Datacore, Data  //
//     / \  | |/ ___| ___ _ __ |___ \   //------------------------------------//
//    / _ \ | | |  _ / _ \ '_ \  __) |  // Klasse TImageManager mit Haupt-    //
//   / ___ \| | |_| |  __/ | | |/ __/   // instanz Data zur Verwaltung der    //
//  /_/   \_\_|\____|\___|_| |_|_____|  // Bilderliste und zugehrige         //
//                                      // Funktionen                         //
////////////////////////////////////////////////////////////////////////////////
unit Datacore;
interface
uses Windows, SysUtils, Classes, Graphics;

// TRotation (diskrete Werte fr die Rotation) und Klartextentsprechungen
type TRotation=(rot0,rot90CW,rot180,rot90CCW);
const Rotation:array[TRotation]of string=('nicht drehen','90 nach rechts',
                                          '180','90 nach links');

// TImageRecord: ein Eintrag aus der Bilderliste
type PImageRecord=^TImageRecord;
     TImageRecord=record
                    // Alles, was das UI wissen mu
                    Thumbnail:TBitmap;       // Thumbnail-Bitmap
                    FileName:string;         // Name der Datei (ohne Pfad)
                    PathName:string;         // Name der Datei (mit Pfad)
                    ImageInfo:string;        // "JPEG 123x456 ..."
                    Active:boolean;          // Hkchen
                    Selected:boolean;        // ausgewhlt (frs UI)
                    Comment:string;          // Bildbeschreibung
                    FileDate:TDateTime;      // Dateidatum
                    EXIFDate:TDateTime;      // EXIF-Datum
                    DisplayDate:string[19];  // Dateidatum als Anzeigetext
                    Rotation:TRotation;      // Drehung
                    Category:integer;        // Kategorie
                    // Alles, was der Kern wissen mu
                    Number:integer;          // Bildnummer
                    Prev,Next:integer;       // Vorgnger- und Nachfolgerzeiger
                    ThumbSX,ThumbSY:integer; // Gre der Thumbnails
                  end;

// Gre der Bilderlisten-Thumbnails                  
const ThumbnailWidth=106;
      ThumbnailHeight=80;

// Komparatorfunktionen zum Vergleich zweier ImageRecords
type TImageRecordComparator=function(const a,b:TImageRecord):integer;
function CompareByName(const a,b:TImageRecord):integer;
function CompareByFileDate(const a,b:TImageRecord):integer;
function CompareByEXIFDate(const a,b:TImageRecord):integer;

// "leerer" Standard-ImageRecord 
const EmptyImageRecord:TImageRecord=(Thumbnail:nil;FileName:'';PathName:'';
      ImageInfo:'';Active:false;Selected:false;Comment:'';FileDate:0;EXIFDate:0;
      DisplayDate:'';Rotation:rot0;Category:0;Number:0;Prev:-1;Next:-1;
      ThumbSX:0;ThumbSY:0);

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////// TImageManager
////////////////////////////////////////////////////////////////////////////////

type TImageManager=class
                   private     
                     FImages:array of TImageRecord;  // Bilderlistendaten
                     FImageMax:integer;  // Index des letzten belegten Eintrags
                     FCategories:array of string;  // Kategoriendaten
                     FSelectedCategory:integer;  // vom UI ausgewhlte Kategorie
                           // (einige Operationen arbeiten nur innerhalb dieser)
                     FStartOfList:integer;
                                         // Index des ersten nummeriertes Bildes
                     FEnlistedImages:integer;  // Anzahl nummerierter Bilder
                     // diverse property-read/write-Funktionen
                     function GetImage(Index:integer):TImageRecord;
                     procedure SetImage(Index:integer; const Value:TImageRecord);
                     function GetImagePtr(Index:integer):PImageRecord;
                     function GetCategory(Index:integer):string;
                     procedure SetCategory(Index:integer; const Value:string);
                   public
                     // Konstruktor/Destruktor
                     constructor Create;
                     destructor Destroy; override;
                     ///// alles, was die Bilderliste betrifft
                     property Images[Index:integer]:TImageRecord
                       read GetImage write SetImage;
                       // Wrapper um FImages
                     property ImagePtr[Index:integer]:PImageRecord
                       read GetImagePtr; default;
                       // Default-Eigenschaft liefert Zeiger(!) auf die FImages-
                       // Eintrge zwecks schnellerem und schreibendem Zugriff
                     function CountImages:integer; // Anzahl Bilder in der Liste
                     property ImageCount:integer read CountImages;
                                              // nur eine andere Schreibweise :)
                     procedure AddImage(const Value:TImageRecord); overload;
                                          // ein Bild per ImageRecord hinzufgen
                     function AddImage:PImageRecord; overload;
                             // ein Bild mit leerem ImageRecord hinzufgen, aber
                             // Zeiger darauf liefern, um spter aufzufllen
                     procedure RemoveSelectedImages;
                         // alle Bilder, die als Selected markiert sind, lschen
                     function FindImage(const PathName:string):integer;
                                                     // Bild per PathName finden
                     procedure SortImages(Comparator:TImageRecordComparator);
                                     // Bilder der aktuellen Kategorie sortieren
                     procedure Rehash;  // Bilder neu nach Kategorien sortieren
                     procedure Renum;  // Zieldateinummerierung neu berechnen
                     property StartOfList:integer read FStartOfList;  // Wrapper
                     property EnlistedImages:integer read FEnlistedImages;  // "
                     procedure ImagesToStrings(Dest:TStrings);
                            // Indexe der Bilder von SelectedCategory nach Liste
                     procedure UnselectAll;  // alle Bilder abwhlen
                     procedure Select(Index:integer);  // ein Bild anwhlen
                     procedure MoveSelectedUp;  // ausgewhlte Bilder nach oben
                     procedure MoveSelectedDown;  // ausgew. Bilder nach unten
                     ///// alles, was die Kategorienliste betrifft
                     property Categories[Index:integer]:string
                       read GetCategory write SetCategory;
                       // Wrapper um FCategories
                     property SelectedCategory:integer
                       read FSelectedCategory write FSelectedCategory;
                       // Wrapper um FSelectedCategory
                     function CountCategories:integer;  // Anzahl der Kategorien
                     property CategoryCount:integer read CountCategories;
                                              // nur eine andere Schreibweise :)
                     procedure AddCategory(const Name:string); // neue Kategorie
                     function RemoveCategory(Index:integer):boolean;
                                                          // Kategorie entfernen
                     function ExchangeCategory(a,b:integer):boolean;
                                           // zwei Kategorien per Index tauschen
                     procedure CategoriesToStrings(Start:integer; Dest:TStrings);
                                         // Kategoriennamen in TStrings kopieren
                   end;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////// Verschiedenes
////////////////////////////////////////////////////////////////////////////////

var Data:TImageManager;  // Hauptinstanz des ImageManagers


// i2s/s2i: en-/decodiert die Indexwerte, die TImageManager.ImagesToList() in
// die TListBox MainForm.CImages hineinschreibt
function i2s(i:integer):string;
function s2i(const s:string):integer;


implementation
uses Main;


///// FImages wird immer nur in $IMAGEBUFFERGRANULARITY gren Blcken erweitert
const IMAGEBUFFERGRANULARITY=16;


///// s2i und i2s codieren die Bilder in zwei Zeichen, jeweils 7 Bit breit
///// (daher die Beschrnkung auf 16383 Bilder);
///// das 8. Bit wird gesetzt, so da ASCII-Werte von 128 bis 255 herauskommen
function i2s(i:integer):string;
begin
  Result:=char(128 OR (i AND 127))+char(128 OR ((i SHR 7) AND 127));
end;
function s2i(const s:string):integer;
begin
  Result:=(ord(s[1]) AND 127) OR ((ord(s[2]) AND 127) SHL 7);
end;


///// Komparatoren
function CompareByName(const a,b:TImageRecord):integer;
begin
  Result:=StrIComp(PChar(a.PathName),PChar(b.PathName));
end;
function CompareByFileDate(const a,b:TImageRecord):integer;
begin
  if a.FileDate<b.FileDate then Result:=-1 else
  if a.FileDate>b.FileDate then Result:=+1 else
                                Result:=0;
end;
function CompareByEXIFDate(const a,b:TImageRecord):integer;
begin
  if a.EXIFDate<b.EXIFDate then Result:=-1 else
  if a.EXIFDate>b.EXIFDate then Result:=+1 else
                                Result:=0;
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////// Initialisierung, Deinitialisierung, Triviales
////////////////////////////////////////////////////////////////////////////////

///// Create: erzeugt eine leere ImageManager-Instanz
constructor TImageManager.Create;
begin
  inherited Create;
  SetLength(FImages,0); FImageMax:=-1;
  SetLength(FCategories,0); FSelectedCategory:=-1;
  FStartOfList:=-1; FEnlistedImages:=0;
end;


///// Destroy: rumt die Instanz auf
destructor TImageManager.Destroy;
var Index:integer;
begin
  // Thumbnails freigeben
  for Index:=0 to FImageMax do
    with FImages[Index] do if Assigned(Thumbnail) then FreeAndNil(Thumbnail);
  // Arrays auflsen
  Finalize(FImages);
  Finalize(FCategories);
  inherited Destroy;
end;


///// GetCategory: liefert den Namen einer bestimmten Kategorie;
///// die Kategorienindexe sind dabei 1-basiert (!); 0 und -1 liefern die
///// Spezialkategoriennamen
function TImageManager.GetCategory(Index:integer):string;
begin
  case Index of
    -1:Result:='(alle)';
    0:Result:='(nicht zugeordnet)';
  else
    if (Index>0) and (Index<=Length(FCategories))
      then Result:=FCategories[Index-1]
      else Result:='';
  end;
end;


///// SetCategory: setzt den Namen einer Kategorie neu
procedure TImageManager.SetCategory(Index:integer; const Value:string);
begin
  if (Index>0) and (Index<=Length(FCategories))
    then FCategories[Index-1]:=Value;
end;


///// GetImage: liefert einen ImageRecord
function TImageManager.GetImage(Index:integer):TImageRecord;
begin
  if (Index>=0) AND (Index<=FImageMax) then
    Result:=FImages[Index]
  else Result:=EmptyImageRecord;
end;


///// SetImage: speichert einen ImageRecord in der Liste ab
procedure TImageManager.SetImage(Index:integer; const Value:TImageRecord);
begin
  if (Index>=0) AND (Index<=FImageMax)
    then FImages[Index]:=Value;
end;


///// GetImagePtr: liefert einen Zeiger auf einen ImageRecord
function TImageManager.GetImagePtr(Index:integer):PImageRecord;
begin
  if (Index>=0) AND (Index<=FImageMax)
    then Result:=@FImages[Index]
    else Result:=nil;
end;


///// CountImages: liefert die Anzahl der Bilder in der Liste
function TImageManager.CountImages:integer;
begin
  Result:=FImageMax+1;
end;


///// CountCategories: liefert die Anzahl der Kategorien
function TImageManager.CountCategories:integer;
begin
  Result:=length(FCategories);
end;


////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////// Bilderverwaltung
////////////////////////////////////////////////////////////////////////////////

///// AddImage(1): legt ein neues Bild an, fllt dessen ImageRecord mit Vorgabe-
///// werten und liefert einen Zeiger darauf zurck (zwecks spterer Befllung)
function TImageManager.AddImage:PImageRecord;
begin
  inc(FImageMax);
  // Puffer vergrern, falls ntig
  if (FImageMax>High(FImages)) then
    SetLength(FImages,length(FImages)+IMAGEBUFFERGRANULARITY);
  // Daten vorbelegen
  FImages[FImageMax]:=EmptyImageRecord;
  Result:=@FImages[FImageMax];
end;


///// AddImage(2): legt ein neues Bild an und schreibt den gegebenen Record
///// direkt dort hinein
procedure TImageManager.AddImage(const Value:TImageRecord);
begin
  AddImage()^:=Value;
end;


///// ImagesToStrings: gibt die Indexe der Bilder aus der gerade aktiven
///// Kategorie (oder allen Kategorien, wenn SelectedCategory="(alle)") in einer
///// TStrings-Liste aus
procedure TImageManager.ImagesToStrings(Dest:TStrings);
var i:integer;
begin
  Dest.Clear;
  for i:=0 to FImageMax do
    if (FSelectedCategory<0) OR (FImages[i].Category=FSelectedCategory) then
      Dest.Add(i2s(i));
end;


///// Rehash: sortiert das FImages[]-Array stabil nach Kategorien
procedure TImageManager.Rehash;
var Temp:TImageRecord;
    a,b:integer;
begin
  // Bubblesort -- graaaaauenhaft langsam, aber wenigstens stabil
  for a:=FImageMax downto 0 do
    for b:=0 to (a-1) do
      if FImages[b].Category>FImages[b+1].Category then begin
        Temp:=FImages[b];
        FImages[b]:=FImages[b+1];
        FImages[b+1]:=Temp;
      end;
end;


///// FindImage: sucht den Index eines Bildes anhand dessen PathName
function TImageManager.FindImage(const PathName:string):integer;
var i:integer;
begin
  for i:=0 to FImageMax do
    if FImages[i].PathName=PathName then begin
      Result:=i;
      exit;
    end;
  Result:=-1;
end;


///// UnselectAll: setzt .Selected aller Bilder auf false
procedure TImageManager.UnselectAll;
var i:integer;
begin
  for i:=0 to FImageMax do
    FImages[i].Selected:=false;
end;


///// Select: setzt .Selected eines Bilds auf true
procedure TImageManager.Select(Index:integer);
begin
  if (Index>=0) AND (Index<=FImageMax) then
    FImages[Index].Selected:=true;
end;


///// RemoveSelectedImages: entfernt Bilder mit .Selected=true aus der Liste
procedure TImageManager.RemoveSelectedImages;
var Source,Dest:integer;
begin
  // Verfahren: Das Array wird mit Source durchgegangen und alle nichtselektier-
  // ten Bilder werden (preinkrementell) nach Dest kopiert
  Dest:=-1;
  for Source:=0 to FImageMax do
    if FImages[Source].Selected then begin
      // Thumbnails gelschter Bilder freigeben
      with FImages[Source] do if Assigned(Thumbnail) then FreeAndNil(Thumbnail);
    end else begin
      inc(Dest);
      if Dest<>Source then FImages[Dest]:=FImages[Source];
    end;
  FImageMax:=Dest;  // ImageMax anpassen
end;


////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////// Kategorienverwaltung
////////////////////////////////////////////////////////////////////////////////

///// AddCategory: fgt eine Kategorie mit gegebenem Namen hinzu
procedure TImageManager.AddCategory(const Name:string);
begin
  // da sich die Kategorien nicht allzu oft ndern, kann hier ruhig das Array
  // so gro sein wie sein Datenbereich
  SetLength(FCategories,length(FCategories)+1);
  FCategories[High(FCategories)]:=Name;
end;


///// CategoriesToStrings: kopiert die Namen der Kategorien (ab einer
///// bestimmten, um "(alle)" und "(unassigned)" zu inkludieren oder eben nicht)
///// in eine TStrings-Liste
procedure TImageManager.CategoriesToStrings(Start:integer; Dest:TStrings);
var i:integer;
begin
  Dest.Clear;
  for i:=Start to length(FCategories) do
    Dest.Add(Categories[i]);
end;


///// ExchangeCategory: tauscht zwei Kategorien aus und pat die Bilderliste an
///// Ergebnis = Erfolgsmeldung
function TImageManager.ExchangeCategory(a,b:integer):boolean;
var Temp:string;
    i:integer;
begin
  Result:=false;  // Pessimismus ist immer gut :)
  // ungltige Eingaben abweisen
  if (a<1) OR (a>length(FCategories)) then exit;
  if (b<1) OR (b>length(FCategories)) OR (a=b) then exit;
  // Kategoriennamen tauschen
  Temp:=FCategories[a-1];
  FCategories[a-1]:=FCategories[b-1];
  FCategories[b-1]:=Temp;
  // Kategorienindexe in der Bilderliste tauschen
  for i:=0 to FImageMax do
    with FImages[i] do
      if Category=a then Category:=b else
      if Category=b then Category:=a;
  // so, _jetzt_ ist's geschafft
  Result:=true;
end;


///// RemoveCategory: lscht eine Kategorie und pat die Bilderliste an
function TImageManager.RemoveCategory(Index:integer):boolean;
var i:integer;
begin
  Result:=false;  // Pessimismus ist immer gut :)
  // ungltige Eingaben abweisen
  if (Index<1) OR (Index>length(FCategories)) then exit;
  // nachfolgende Kategorien nachrcken lassen
  for i:=Index to High(FCategories) do
    FCategories[i-1]:=FCategories[i];
  // Kategorienarray verkleinern
  SetLength(FCategories,length(FCategories)-1);
  // Bilderliste anpassen:
  for i:=0 to FImageMax do
    with FImages[i] do
      // alle Verweise zur gelschten Kategorie auf "(unassigned)" umbiegen
      if Category=Index then Category:=0 else
      // alle Verweise auf nachfolgende Kategorien nachrutschen lassen
      if Category>Index then dec(Category);
  // so, _jetzt_ ist's geschafft
  Result:=true;
end;


////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////// komplexe Operationen
////////////////////////////////////////////////////////////////////////////////

///// MoveSelectedUp: bewegt alle markierten Bilder der aktiven Kategorie einen
///// Eintrag nach oben
procedure TImageManager.MoveSelectedUp;
var Index,Start:integer;
    Temp:TImageRecord;
begin
  // erster nach oben bewegbarer Eintrag ist der zweite (Index 1)
  Start:=1;
  // bei Einschrnkung auf eine Kategorie erstes Bild dieser Kategorie finden
  if FSelectedCategory>=0 then
    for Index:=0 to FImageMax do
      if FImages[Index].Category=FSelectedCategory then begin
        Start:=Index+1;  // ersten bewegbaren Eintrag entsprechend anpassen
        break;
      end;
  // alle folgenden Eintrge mit ihren Vorgngern austauschen, wenn dieser nicht
  // markiert ist und folglich schon bewegt wurde
  for Index:=Start to FImageMax do
    if FImages[Index].Selected AND not(FImages[Index-1].Selected) then begin
      Temp:=FImages[Index-1];
      FImages[Index-1]:=FImages[Index];
      FImages[Index]:=Temp;
    end;
end;


///// MoveSelectedDown: bewegt alle markierten Bilder der aktiven Kategorie
///// einen Eintrag nach unten
procedure TImageManager.MoveSelectedDown;
var Index,Stop:integer;
    Temp:TImageRecord;
begin
  // letzer nach unten bewegbarer Eintrag ist der vorletzte
  Stop:=FImageMax-1;
  // bei Einschrnkung auf eine Kategorie letztes Bild dieser Kategorie finden
  if FSelectedCategory>=0 then
    for Index:=FImageMax downto 1 do
      if FImages[Index].Category=FSelectedCategory then begin
        Stop:=Index-1;  // letzten bewegbaren Eintrag entsprechend anpassen
        break;
      end;
  // alle folgenden Eintrge in Rckwrtsreihenfolge mit ihren Nachfolgern
  // austauschen, wenn dieser nicht markiert ist und folglich schon bewegt wurde
  for Index:=Stop downto 0 do
    if FImages[Index].Selected AND not(FImages[Index+1].Selected) then begin
      Temp:=FImages[Index+1];
      FImages[Index+1]:=FImages[Index];
      FImages[Index]:=Temp;
    end;
end;


///// SortImages: sortiert die Bilder der aktiven Kategorie mit dem angegebenen
///// Komparator
procedure TImageManager.SortImages(Comparator:TImageRecordComparator);
var Start,Stop:integer;
    a,b,Index:integer;
    Temp:TImageRecord;
begin
  // prinzipiell alle Bilder sortieren ...
  Start:=0; Stop:=FImageMax;
  // ... auer, es ist eine Kategorie gewhlt
  if FSelectedCategory>=0 then begin
    // erstes Bild der Kategorie finden
    for Index:=0 to FImageMax do
      if FImages[Index].Category=FSelectedCategory then
        begin Start:=Index; break; end;
    // letztes Bild der Kategorie finden
    for Index:=FImageMax downto 0 do
      if FImages[Index].Category=FSelectedCategory then
        begin Stop:=Index; break; end;
  end;
  // zwischen diesen beiden Schranken mit Bubblesort-2 sortieren
  for a:=Start to (Stop-1) do
    for b:=(a+1) to Stop do
      if Comparator(FImages[a],FImages[b])>0 then begin
        Temp:=FImages[a];
        FImages[a]:=FImages[b];
        FImages[b]:=Temp;
      end;
end;


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////// neue Nummerierung
////////////////////////////////////////////////////////////////////////////////

///// Renum: alle Bilder neu nummerieren
procedure TImageManager.Renum;
type TNumberingMode=(nmuIgnore,nmuStart,nmuEnd);
var Index,Last,Start,Count:integer;
    NumberingMode:TNumberingMode;
begin
  // Nummerierungsmodus bestimmen
  if MainForm.CUncatAtStart.Checked then NumberingMode:=nmuStart else
  if MainForm.CUncatAtEnd.Checked   then NumberingMode:=nmuEnd   else
                                         NumberingMode:=nmuIgnore;
  // Nummerierung lschen
  for Index:=0 to FImageMax do
    with FImages[Index] do begin
      Number:=0; Prev:=-1; Next:=-1;
      ThumbSX:=0; ThumbSY:=0;
    end;
  FStartOfList:=-1;
  // erstes eingeordnetes Bild bestimmen
  if NumberingMode=nmuStart then
    Start:=0
  else begin
    Start:=FImageMax+1;
    for Index:=0 to FImageMax do
      if FImages[Index].Category>0 then begin
        Start:=Index;
        break;
      end;
  end;
  // Bilder nummerieren
  Last:=-1; Count:=0;
  for Index:=Start to FImageMax do
    with FImages[Index] do
      if Active then begin
        inc(Count); Number:=Count;
        if Last<0 then FStartOfList:=Index
                  else FImages[Last].Next:=Index;
        Prev:=Last;
        Last:=Index;
      end;
  // evtl. nicht eingeordnete Bilder ans Ende herannummerieren
  if NumberingMode=nmuEnd then
    for Index:=0 to (Start-1) do
      with FImages[Index] do
        if Active then begin
          inc(Count); Number:=Count;
          if Last<0 then FStartOfList:=Index
                    else FImages[Last].Next:=Index;
          Prev:=Last;
          Last:=Index;
        end;
  FEnlistedImages:=Count;
end;

end.
