////////////////////////////////////////////////////////////////////////////////
//      _    _  ____            ____    // Preview.pas: unit Preview          //
//     / \  | |/ ___| ___ _ __ |___ \   //------------------------------------//
//    / _ \ | | |  _ / _ \ '_ \  __) |  // Vorschaufenster (TPreviewForm)     //
//   / ___ \| | |_| |  __/ | | |/ __/   //                                    //
//  /_/   \_\_|\____|\___|_| |_|_____|  //                                    //
//                                      //                                    //
////////////////////////////////////////////////////////////////////////////////
unit Preview;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls;

type
  TPreviewForm = class(TForm)
    PBX: TPaintBox;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure PBXPaint(Sender: TObject);
    procedure FormMouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormMouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    MyBMP:TBitmap;  // lokale Kopie der anzuzeigenden Bitmap
    Zoom:integer;  // Zoomfaktor (positiv=Vergrerung, negativ=Verkleinerung)
    BaseCaption:string;  // Hauptteil des Fenstertitels
    procedure SetZoom(NewZoom:integer);  // Zoomfaktor ndern
    procedure UpdateCaption;  // Fenstertitel aktualisieren
  public
    { Public declarations }
    procedure Show(YourBMP:TBitmap; const BaseCaption:string); overload;
           // berladene Show-Methode, die zustzlich eine neue Bitmap und einen
           // neuen Fenstertitel bernimmt
  end;

var
  PreviewForm: TPreviewForm;

implementation

{$R *.dfm}


///// PreviewForm.OnCreate: harmloser Konstruktur
procedure TPreviewForm.FormCreate(Sender: TObject);
begin
  MyBMP:=nil;
end;


///// PreviewForm.OnDestroy: harmloser Destruktor
procedure TPreviewForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  if Assigned(MyBMP) then FreeAndNil(MyBMP);
end;


///// PreviewForm.Show: Fenster anzeigen, vorher neue Bitmap und neuen
///// Fenstertitel bernehmen
procedure TPreviewForm.Show(YourBMP:TBitmap; const BaseCaption:string);
begin
  if Assigned(YourBMP) then begin
    // Bitmap bernehen
    if not Assigned(MyBMP) then MyBMP:=TBitmap.Create;
    MyBMP.Assign(YourBMP);
    // Gre der PaintBox setzen
    SetZoom(0);
    PBX.Visible:=true;
    // Fenster gnstig positionieren
    ClientWidth:=MyBMP.Width;
    ClientHeight:=MyBMP.Height;
    if Left+Width>Monitor.BoundsRect.Right then begin
      Left:=Monitor.BoundsRect.Right-Width;
      if Left<0 then begin Left:=0; Width:=Monitor.Width; end;
    end;
    if Top+Height>Monitor.BoundsRect.Bottom then begin
      Top:=Monitor.BoundsRect.Bottom-Height;
      if Top<0 then begin Top:=0; Height:=Monitor.Height; end;
    end;
  end else begin
    // keine Bitmap bergeben -> keine Bitmap anzeigen
    if Assigned(MyBMP) then FreeAndNil(MyBMP);
    PBX.Visible:=false;
  end;
  // noch ein paar Details ...
  Self.BaseCaption:=BaseCaption;
  UpdateCaption;
  if Visible then Invalidate else Show;
  SetFocus;
end;


///// PBX.OnPaint: Bitmap (entsprechend vergrert) neu zeichnen
procedure TPreviewForm.PBXPaint(Sender: TObject);
begin
  if Assigned(MyBMP) then
    if Zoom=0 then PBX.Canvas.Draw(0,0,MyBMP)
              else PBX.Canvas.StretchDraw(PBX.ClientRect,MyBMP);
end;


///// PreviewForm.OnMouseWheelDown: Zoomfaktor senken
procedure TPreviewForm.FormMouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  SetZoom(Zoom-1);
end;


///// PreviewForm.OnMouseWheelUp: Zoomfaktor erhhen
procedure TPreviewForm.FormMouseWheelUp(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  SetZoom(Zoom+1);
end;


///// SetZoom: Zoomfaktor ndern, Gre der PaintBox entsprechend anpassen
procedure TPreviewForm.SetZoom(NewZoom:integer);
begin
  // Zoom auf sinnvolle Werte begrenzen
  if NewZoom>31 then NewZoom:=31;
  if NewZoom<-31 then NewZoom:=-31;
  Zoom:=NewZoom;
  // PaintBox-Gre setzen
  if Zoom>=0 then begin
    PBX.Width:=MyBMP.Width*(Zoom+1);
    PBX.Height:=MyBMP.Height*(Zoom+1);
  end else begin
    PBX.Width:=MyBMP.Width DIV (1-Zoom);
    PBX.Height:=MyBMP.Height DIV (1-Zoom);
  end;
  // Fenstertitel anpassen
  UpdateCaption;
end;


///// UpdateCaption: neuen Fenstertitel aus Basistitel und Zoomfaktor bauen
procedure TPreviewForm.UpdateCaption;
begin
  if Zoom>=0
    then Caption:=BaseCaption+' ['+IntToStr(Zoom+1)+':1]'
    else Caption:=BaseCaption+' [1:'+IntToStr(1-Zoom)+']';
end;


///// PreviewForm.OnKeyPress: Plus- und Minustaste zum Zoomen verwenden, fr all
///// jene, die kein Mausrad haben
procedure TPreviewForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key='+' then SetZoom(Zoom+1);
  if Key='-' then SetZoom(Zoom-1);
end;

end.
