////////////////////////////////////////////////////////////////////////////////
//      _    _  ____            ____    // Process.pas: unit Process          //
//     / \  | |/ ___| ___ _ __ |___ \   //------------------------------------//
//    / _ \ | | |  _ / _ \ '_ \  __) |  // Routinen zur Erzeugung eines       //
//   / ___ \| | |_| |  __/ | | |/ __/   // Albums im Batchvorgang; Konstruk-  //
//  /_/   \_\_|\____|\___|_| |_|_____|  // tion der HTML-Seiten               //
//                                      //                                    //
////////////////////////////////////////////////////////////////////////////////
unit Process;
interface

procedure DoIt;  // die einzige exportierte Routine :)

implementation
uses Windows, SysUtils, Forms, Graphics,
     MLParse, EXIF, Datacore, Core, Main;


////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////// HTML-Tools
////////////////////////////////////////////////////////////////////////////////

///// EXIFCallback: Callback-Routine fr die EXIF-Daten-Auswerdung -- schreibt
///// einfach die EXIF-Daten in die geffnete HTML-Datei als Tabelle ein
var TXT:System.Text;  // mu leider global sein :(
procedure EXIFCallback(TagID,Value:integer; const TagName,Content:string);
begin
  if copy(TagName,1,5)='Tag0x' then exit;
  if copy(Content,1,11)='Unknown (0x' then exit;
  writeln(TXT,'<tr><td align="left"><nobr><b>&nbsp;&nbsp;',MLCharEncode(TagName),'</b></nobr></td>',
              '<td align="left"><nobr>&nbsp;',MLCharEncode(Content),'</nobr></td></tr>');
end;


///// rm_f: rm -f halt :)
procedure rm_f(const Path:string);
var SR:TSearchRec; Cont:integer;
begin
  Cont:=FindFirst(Path,faAnyFile,SR);
  while Cont=0 do begin
    DeleteFile(ExtractFilePath(Path)+SR.Name);
    Cont:=FindNext(SR);
  end;
  FindClose(SR);
end;


///// HTMLhead: schreibt den Header einer HTML-Datei, sofern gewnscht
procedure HTMLhead(var TXT:System.Text; const Title:string);
begin
  writeln(TXT,'<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">');
  writeln(TXT,'<html><head>');
  writeln(TXT,'<meta http-equiv="Content-Type" content="text/html; charset=windows-1250">');
  if MyVersion=''
    then writeln(TXT,'<meta http-equiv="Generator" content="AlGen2">')
    else writeln(TXT,'<meta http-equiv="Generator" content="AlGen2 Version ',MyVersion,'">');
  writeln(TXT,'<title>',MLCharEncode(Title),'</title>');
  writeln(TXT,'</head><body>');
  writeln(TXT,'<!--SOT-->');
end;


///// HTMLfoot: schreibt den Fu einer HTML-Datei, sofern gewnscht
procedure HTMLfoot(var TXT:System.Text);
begin
  writeln(TXT,'<!--EOT-->');
  writeln(TXT,'</body></html>');
end;


///// ProposeCaption: liefert als Bildbeschreibung entweder die eingegebene
///// Beschreibung (Comment) oder extrahiert den Dateivornamen aus FileName
function ProposeCaption(const FileName,Comment:string):string;
begin
  if Comment='' then Result:=copy(FileName,1,LastDelimiter('.',FileName)-1)
                else Result:=Comment;
end;


////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////// DoIt: HTML-Erzeugung
////////////////////////////////////////////////////////////////////////////////

///// Kopf von DoIt
procedure DoIt;
var BaseDir,PreviewDir,ThumbDir:string;  // expandierte Verzeichnisnamen
    PreviewRef,ThumbRef:string;  // HTML-Link-Pfade fr Bilder
    Verbose,GenThumbs,GenIndex,GenGallery,GenInfo:boolean;  // modus operandi

///// WriteIndex: schreibt die HTML-Thumbnailbersicht in die angegebene Datei
procedure WriteIndex(const FileName:string);
var LastCategory,Index,Countdown:integer;
    LinkTo:string;
begin
  // Datei ffnen
  AssignFile(TXT,FileName);
  Rewrite(TXT);
  HTMLhead(TXT,MainForm.ETitle.Text);
  writeln(TXT);
  writeln(TXT,'<h1>',MLCharEncode(MainForm.ETitle.Text),'</h1>');
  // bersicht erzeugen
  Countdown:=MainForm.CThumbsPerLine.Position;
  LastCategory:=0;
  Index:=Data.StartOfList;
  while Index>=0 do with Data[Index]^ do begin
    // Kategorienbergang -> berschrift schreiben
    if Category<>LastCategory then begin
      writeln(TXT);
      writeln(TXT,'<h2>',MLCharEncode(Data.Categories[Category]),'</h2>');
      LastCategory:=Category;
      Countdown:=MainForm.CThumbsPerLine.Position;
    end;
    // einen Bildlink setzen (mit jeder Menge Sonderfallbehandlung)
    if GenGallery then LinkTo:=BuildFileName(Number,MainForm.EfnHTML.Text)+'.htm'
                  else LinkTo:=PreviewRef+BuildFileName(Number,MainForm.EfnPreview.Text)+'.jpg';
    if GenGallery then write(TXT,'<a name="',BuildNumber(Number),'">');
    write(TXT,'<a href="',LinkTo,'"><img src="',ThumbRef,
      BuildFileName(Number,MainForm.EfnThumbnails.Text),'.jpg" width="',ThumbSX,
      '" height="',ThumbSY,'" border="0" alt="',
      MLCharEncode(ProposeCaption(FileName,Comment)),
      '" title="',MLCharEncode(ProposeCaption(FileName,Comment)),'" /></a>');
    if GenGallery then writeln(TXT,'</a>')
                  else writeln(TXT);
    // ggf. einen Zeilenumbruch einfgen
    dec(Countdown);
    if Countdown=0 then begin
      writeln(TXT,'<br clear="all" />');
      Countdown:=MainForm.CThumbsPerLine.Position;
    end;
    // weiter mit dem nchsten Bild
    Index:=Next;
  end;
  // Ende
  writeln(TXT);
  HTMLfoot(TXT);
  CloseFile(TXT);
end;


///// WriteGallery: schreibt eine HTML-Galerien- oder Detailseite
procedure WriteGallery(const FileName:string; const ImageRec:TImageRecord;
                       Details:boolean; SX,SY:integer);
  // LinkTo: gibt einen Link auf Bild Nummer $Index aus; ist $Index ungltig
  // (weil auerhalb der Liste), so wird der Link nicht gesetzt, sondern nur
  // seine Beschriftung
  procedure LinkTo(Index:integer; const Caption:string);
  begin
    if Index>=0 then if Details
      then write(TXT,'<a href="',BuildFileName(Data[Index]^.Number,MainForm.EfnInfo.Text),'.htm">')
      else write(TXT,'<a href="',BuildFileName(Data[Index]^.Number,MainForm.EfnHTML.Text),'.htm">');
    write(TXT,Caption);
    if Index>=0 then write(TXT,'</a>');
  end;
begin
  // Datei ffnen
  AssignFile(TXT,FileName);
  Rewrite(TXT);
  HTMLhead(TXT,ImageRec.Comment);
  // Tabelle mit Kopfzeile einleiten
  if MainForm.CFullWidth.Checked
    then write(TXT,'<table width="100%"')
    else write(TXT,'<table');
  writeln(TXT,' align="center" border="0" cellpadding="0" cellspacing="3" bgcolor="silver"><tr>');
  write(TXT,'<td align="left" valign="top"><nobr>');
  LinkTo(ImageRec.Prev,'&lt;&lt; Zurck'); writeln(TXT,'</nobr></td>');
  writeln(TXT,'<th valign="top">',
      MLCharEncode(ProposeCaption(ImageRec.FileName,ImageRec.Comment)),'</th>');
  write(TXT,'<td align="right" valign="top"><nobr>');
  LinkTo(ImageRec.Next,'Weiter &gt;&gt;'); writeln(TXT,'</nobr></td>');
  // EXIF-Informationen einbinden
  if Details then begin
    writeln(TXT,'<td rowspan="3" valign="top"><table border="0" cellpadding="0" cellspacing="0">');
    EXIFParse(ImageRec.PathName,EXIFCallback);
    writeln(TXT,'</table></td>');
  end;
  // das Bild selbst
  writeln(TXT,'</tr><tr><td colspan="3" align="center" valign="top"><img src="',
          PreviewRef,BuildFileName(ImageRec.Number,MainForm.EfnPreview.Text),
          '.jpg" width="',SX,'" height="',SY,'" border="0" /></td></tr><tr>');
  // Fuzeile: Link zum Index und Leerzelle
  writeln(TXT,'<td align="left"><nobr><a href="',MainForm.EfnIndex.Text,'.htm#',
          BuildNumber(ImageRec.Number),'">&lt;- Index</nobr></a></td>');
  writeln(TXT,'<td>&nbsp;</td>');
  // Link zu den EXIF-Details bzw. von diesen zurck (oder halt gar keiner)
  if Details then
    writeln(TXT,'<td align="right"><nobr><a href="',
            BuildFileName(ImageRec.Number,MainForm.EfnHTML.Text),
            '.htm">Details &lt;-</a></nobr></td>')
  else if GenInfo then
    writeln(TXT,'<td align="right"><nobr><a href="',
            BuildFileName(ImageRec.Number,MainForm.EfnInfo.Text),
            '.htm">Details -&gt;</a></nobr></td>')
  else
    writeln(TXT,'<td>&nbsp;</td>');
  // Datei schlieen
  writeln(TXT,'</tr></table>');
  HTMLfoot(TXT);
  CloseFile(TXT);
end;


////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////// Hauptteil von DoIt
////////////////////////////////////////////////////////////////////////////////

var Count,Index,Total,Success:integer;  // diverse Zhler
    CurrentFile:string;  // Name der aktuell zu erzeugenden Zieldatei
    ImageRec:PImageRecord;
    RawImage,ScaledImage:TBitmap;  // Zwischenbilder
    Error:boolean;  // Fehlerflag fr aktuelles Bild
    T1,T2:cardinal;  // zur Zeitnahme
    PreviewSX,PreviewSY:integer;  // Gre des erzeugten Vorschaubildes
                                  // (fr die HTML-Generierung gecachet)  

begin
  // Pfade ermitteln
  BaseDir:=IncludeTrailingPathDelimiter(MainForm.EDestDir.Text);
  if MainForm.CdirPreview.Checked then begin
    PreviewDir:=BaseDir+MainForm.EdirPreview.Text+'\';
    PreviewRef:=MainForm.EdirPreview.Text+'/';
  end else begin
    PreviewDir:=BaseDir;
    PreviewRef:='';
  end;
  if MainForm.CdirThumbnails.Checked then begin
    ThumbDir:=BaseDir+MainForm.EdirThumbnails.Text+'\';
    ThumbRef:=MainForm.EdirThumbnails.Text+'/';
  end else begin
    ThumbDir:=BaseDir;
    ThumbRef:='';
  end;

  // Betriebsmodus ermitteln
  Index:=MainForm.COutputHTML.ItemIndex;
  GenThumbs:=(Index>=1);
  GenIndex:=(Index>=2);
  GenGallery:=(Index>=3);
  GenInfo:=(Index>=4);

  // Initialisierung
  Data.Renum;
  Total:=Data.EnlistedImages;
  MainForm.SelectOp(nil);
  Log('');
  Log('/Erzeuge Album aus '+IntToStr(Total)+' Bildern');
  T1:=GetTickCount();

  // rm -rf, wenn gewnscht
  if MainForm.Crmrf.Checked then begin
    if Verbose then
      Log('*Lsche vorhandene Dateien aus '+ExcludeTrailingPathDelimiter(BaseDir));
    rm_f(BaseDir+'*.htm');
    rm_f(BaseDir+'*.jpg');
    rm_f(PreviewDir+'*.jpg');
    rm_f(ThumbDir+'*.jpg');
    if not Verbose then
      Log('*Alte Dateien gelscht');
  end;

  // Verzeichnisse anlegen
  if (not(DirectoryExists(PreviewDir)) AND not(CreateDir(PreviewDir))) OR
     (not(DirectoryExists(ThumbDir)) AND not(CreateDir(ThumbDir))) then
  begin
    Log('-Kann Zielverzeichnisse nicht anlegen!');
    Log('');
    MainForm.SelectOp(MainForm.PLog);
    exit;
  end;

  // Bilder erzeugen ///////////////////////////////////////////////////////////
  RawImage:=nil; ScaledImage:=nil;
  Success:=0;
  Count:=0; Index:=Data.StartOfList;
  while Index>=0 do begin inc(Count);
    Verbose:=MainForm.CVerbose.Checked;
    ImageRec:=Data[Index]; Error:=false;
    if Verbose then
      Log('*['+IntToStr(Count)+'/'+IntToStr(Total)+'] Bearbeite '+ImageRec^.PathName);
    PreviewSX:=0; PreviewSY:=0;
    // Bild einlesen
    RawImage:=PreprocessImage(ImageRec^);
    if not Assigned(RawImage) then begin
      Error:=true;
      if Verbose then
        Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Einlesen');
    end else begin
      // Preview erzeugen
      CurrentFile:=PreviewDir+BuildFileName(ImageRec^.Number,MainForm.EfnPreview.Text)+'.jpg';
      if Verbose then
        Log('_['+IntToStr(Count)+'/'+IntToStr(Total)+'] Erzeuge Vorschaubild: '+CurrentFile);
      ScaledImage:=ScaleImage(ImageRec^,RawImage,itPreview);
      if not Assigned(ScaledImage) then begin
        Error:=true;
        if Verbose then
          Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Berechnen');
      end else begin
        if not CompressImage(ScaledImage,CurrentFile,itPreview) then begin
          Error:=true;
          if Verbose then
            Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Schreiben');
        end else begin
          PreviewSX:=ScaledImage.Width;
          PreviewSY:=ScaledImage.Height;
        end;
        FreeAndNil(ScaledImage);
      end;
      // Thumbnail erzeugen
      if GenThumbs then begin
        CurrentFile:=ThumbDir+BuildFileName(ImageRec^.Number,MainForm.EfnThumbnails.Text)+'.jpg';
        if Verbose then
          Log('_['+IntToStr(Count)+'/'+IntToStr(Total)+'] Erzeuge Thumbnail: '+CurrentFile);
        ScaledImage:=ScaleImage(ImageRec^,RawImage,itThumbnail);
        if not Assigned(ScaledImage) then begin
          Error:=true;
          if Verbose then
            Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Berechnen');
        end else begin
          if not CompressImage(ScaledImage,CurrentFile,itThumbnail) then begin
            Error:=true;
            if Verbose then
              Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Schreiben');
          end else begin
            ImageRec^.ThumbSX:=ScaledImage.Width;
            ImageRec^.ThumbSY:=ScaledImage.Height;
          end;
          FreeAndNil(ScaledImage);
        end;
      end;
      FreeAndNil(RawImage);
      // Galeriendatei schreiben
      if GenGallery then begin
        CurrentFile:=BaseDir+BuildFileName(ImageRec^.Number,MainForm.EfnHTML.Text)+'.htm';
        if Verbose then
          Log('_['+IntToStr(Count)+'/'+IntToStr(Total)+'] Schreibe Galeriendatei: '+CurrentFile);
        try
          WriteGallery(CurrentFile,ImageRec^,false,PreviewSX,PreviewSY);
        except
          Error:=true;
          if Verbose then
            Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Schreiben');
        end;
      end;
      // Details-Datei schreiben
      if GenInfo then begin
        CurrentFile:=BaseDir+BuildFileName(ImageRec^.Number,MainForm.EfnInfo.Text)+'.htm';
        if Verbose then
          Log('_['+IntToStr(Count)+'/'+IntToStr(Total)+'] Schreibe Details-Datei: '+CurrentFile);
        try
          WriteGallery(CurrentFile,ImageRec^,true,PreviewSX,PreviewSY);
        except
          Error:=true;
          if Verbose then
            Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler beim Schreiben');
        end;
      end;
    end;
    // Ende
    if not Verbose then
      if Error then
        Log('-['+IntToStr(Count)+'/'+IntToStr(Total)+'] Fehler: '+ImageRec^.PathName)
      else
        Log('*['+IntToStr(Count)+'/'+IntToStr(Total)+'] '+ImageRec^.PathName);
    if not Error then inc(Success);
    Index:=ImageRec^.Next;
    Application.ProcessMessages; if AbortOp then break;
  end;

  // Index erzeugen ////////////////////////////////////////////////////////////
  if GenIndex then begin
    CurrentFile:=BaseDir+MainForm.EfnIndex.Text+'.htm';
    if Verbose then Log('*Schreibe Index: '+CurrentFile);
    try
      WriteIndex(CurrentFile);
      if not Verbose then Log('*Thumbnail-bersicht geschrieben');
    except
      Log('-Fehler beim Schreiben der Thumbnail-bersicht');
    end;
  end;

  // Ende
  if AbortOp then Log('-Vorgang vom Benutzer abgebrochen');
  T2:=GetTickCount();
  Log('+'+IntToStr(Success)+' von '+IntToStr(Total)+
      ' Bildern erfolgreich umgewandelt in '+IntToStr((T2-T1) DIV 1000)+' Sekunden');
  Log('');
  MainForm.SelectOp(MainForm.PLog);
end;

end.
