unit RotateBMP;
// BMP Rotator Toolkit (C) 2003 Martin J. Fiedler <martin.fiedler@gmx.net>
// This is free software. You may freely redistribute and modify it.
interface
uses SysUtils, Graphics;

function Rotate0(Source:TBitmap):TBitmap;
function Rotate90CW(Source:TBitmap):TBitmap;
function Rotate90CCW(Source:TBitmap):TBitmap;
function Rotate180(Source:TBitmap):TBitmap;

// Each of these functions take an existing TBitmap instance as an argument and
// rotate a newly-created, rotated copy of them. Note that the both the Source
// and the resulting bitmap are set to a PixelFormat of pf24bit.

// The algorithm is TBitmap.ScanLine-based, so it is usually very fast. Rotating
// an 800x600 Bitmap on a 450MHz Celeron (Coppermine-2) is done in 50 msec!

implementation

function Rotate0(Source:TBitmap):TBitmap;
begin
  Result:=TBitmap.Create;
  Result.Assign(Source);
end;

function Rotate90CW(Source:TBitmap):TBitmap;
var U,V,ILO:integer;
    MaxU,MaxV:integer;
    vR,vG,vB:byte;
    Ls,Ld:PByteArray;
    Ps,Pd:^byte;
    LCache:array of PByteArray;
begin
  with Source do begin
    PixelFormat:=pf24bit;
    MaxU:=Width-1;
    MaxV:=Height-1;
  end;
  Result:=TBitmap.Create;
  with Result do begin
    PixelFormat:=pf24bit;
    Width:=MaxV+1;
    Height:=MaxU+1;
  end;
  SetLength(LCache,MaxV+1);
  for V:=0 to MaxV do
    LCache[V]:=Source.ScanLine[V];
  ILO:=0;
  for U:=0 to MaxU do begin
    Ld:=Result.ScanLine[U]; Pd:=@Ld[MaxV*3+2];
    for V:=0 to MaxV do begin
      Ls:=LCache[V]; Ps:=@Ls[ILO];
      vB:=Ps^; inc(Ps);
      vG:=Ps^; inc(Ps);
      vR:=Ps^;
      Pd^:=vR; dec(Pd);
      Pd^:=vG; dec(Pd);
      Pd^:=vB; dec(Pd);
    end;
    inc(ILO,3);
  end;
  Finalize(LCache);
end;

function Rotate90CCW(Source:TBitmap):TBitmap;
var U,V,ILO:integer;
    MaxU,MaxV:integer;
    vR,vG,vB:byte;
    Ls,Ld:PByteArray;
    Ps,Pd:^byte;
    LCache:array of PByteArray;
begin
  with Source do begin
    PixelFormat:=pf24bit;
    MaxU:=Width-1;
    MaxV:=Height-1;
  end;
  Result:=TBitmap.Create;
  with Result do begin
    PixelFormat:=pf24bit;
    Width:=MaxV+1;
    Height:=MaxU+1;
  end;
  SetLength(LCache,MaxV+1);
  for V:=0 to MaxV do
    LCache[V]:=Source.ScanLine[V];
  ILO:=0;
  for U:=0 to MaxU do begin
    Ld:=Result.ScanLine[MaxU-U]; Pd:=@Ld[0];
    for V:=0 to MaxV do begin
      Ls:=LCache[V]; Ps:=@Ls[ILO];
      vB:=Ps^; inc(Ps);
      vG:=Ps^; inc(Ps);
      vR:=Ps^;
      Pd^:=vB; inc(Pd);
      Pd^:=vG; inc(Pd);
      Pd^:=vR; inc(Pd);
    end;
    inc(ILO,3);
  end;
  Finalize(LCache);
end;

function Rotate180(Source:TBitmap):TBitmap;
var X,Y:integer;
    MaxX,MaxY:integer;
    vR,vG,vB:byte;
    Ls,Ld:PByteArray;
    Ps,Pd:^byte;
begin
  with Source do begin
    PixelFormat:=pf24bit;
    MaxX:=Width-1;
    MaxY:=Height-1;
  end;
  Result:=TBitmap.Create;
  with Result do begin
    PixelFormat:=pf24bit;
    Width:=MaxX+1;
    Height:=MaxY+1;
  end;
  for Y:=0 to MaxY do begin
    Ls:=Source.ScanLine[Y]; Ps:=@Ls[MaxX*3+2];
    Ld:=Result.ScanLine[MaxY-Y]; Pd:=@Ld[0];
    for X:=MaxX downto 0 do begin
      vR:=Ps^; dec(Ps);
      vG:=Ps^; dec(Ps);
      vB:=Ps^; dec(Ps);
      Pd^:=vB; inc(Pd);
      Pd^:=vG; inc(Pd);
      Pd^:=vR; inc(Pd);
    end;
  end;
end;

end.
